/*
								+----------------------------------+
								|                                  |
								|    ***   Socket classes   ***    |
								|                                  |
								|   Copyright  -tHE SWINe- 2006   |
								|                                  |
								|             Socket.h             |
								|                                  |
								+----------------------------------+
*/

#ifndef __SOCKET_INCLUDED
#define __SOCKET_INCLUDED

#ifdef WIN32

#include <winsock.h>

/*
 *	class CWinSock
 *		- class for proper initialization and shutdown of winsock 2.0
 *		- note there is no such thing under linux
 */
class CWinSock {
protected:
	bool m_b_ready;

public:
	CWinSock(); // startup
	~CWinSock(); // cleanup

	bool b_Ready(); // ready and supported?
};

#endif //WIN32

/*
 *	struct TAddress
 *		- socket address container for windows / linux
 */
struct TAddress {
public:
#ifdef WIN32
	typedef SOCKADDR_IN TAddrStruct;
#else //WIN32
	typedef struct sockaddr_in TAddrStruct;
#endif //WIN32

protected:

	bool b_valid;
	TAddrStruct t_address;

public:
	/*
	 *	TAddress::TAddress()
	 *		- default constructor
	 */
	inline TAddress() :b_valid(0) {}

	/*
	 *	TAddress::TAddress(const char *p_s_host_name, int n_port);
	 *		- constructor with host name and port
	 *		- note in case host name is 0, INADDR_ANY is used
	 *		- in case host name is not 0, gethostbyname is called, which may fail
	 *		  so the address validity flag is set accordingly (call b_Valid())
	 */
	TAddress(const char *p_s_host_name, int n_port);

	/*
	 *	inline bool b_Valid() const
	 *		- returns address validity flag
	 *		- note this flag is set by the constructor only and be no further manipulated
	 */
	inline bool b_Valid() const { return b_valid; }

	/*
	 *	inline const TAddrStruct &t_Address() const
	 *		- returns network address in form the api uses
	 */
	inline const TAddrStruct &t_Address() const { return t_address; }

	/*
	 *	inline TAddrStruct &t_Address()
	 *		- returns network address in form the api uses
	 */
	inline TAddrStruct &t_Address() { return t_address; }
};

/*
 *	class CSocket
 *		- common class for socket
 *		- note there is no meaning in creating instances of this class
 *		  use CStreamSocket for stream-based sockets or CDatagramSocket
 *		  for datagram based sockets
 */
class CSocket {
protected:
#ifdef WIN32
	static CWinSock m_startup;
	SOCKET m_n_socket;
#else //WIN32
	int m_n_socket;
#endif //WIN32

public:
	/*
	 *	CSocket::CSocket()
	 *		- default constructor, sets no socket
	 */
	CSocket();

	/*
	 *	CSocket::~CSocket()
	 *		- default destructor, releases socket in case it was created
	 */
	~CSocket();

	/*
	 *	bool CSocket::b_State() const
	 *		- returns true in case socket was succesfuly created, otherwise false
	 */
	bool b_State() const;

	/*
	 *	bool CSocket::SetSockOpt(int n_level, int n_name, const void *p_value, int n_value_size)
	 *		- sets socket options
	 *		- returns true in case function succeeded, otherwise false
	 *		- always returns false in case no socket was created
	 */
	bool SetSockOpt(int n_level, int n_name, const void *p_value, int n_value_size);

	/*
	 *	bool CSocket::Close()
	 *		- closes socket
	 *		- in case socket was opened returns true. in case socket was closed already
	 *		  (or has never been created) returns false
	 */
	bool Close();

	/*
	 *	bool CSocket::SetNonBlocking()
	 *		- sets socket non-blocking
	 *		- returns true in case it was succesfully completed, otherwise false
	 *		- always returns false in case no socket was created
	 *		- note in case one of p_Accept, Connect, n_Read or n_Write fails,
	 *		  you have to check b_WouldBlock() to see wheter that was an error or
	 *		  it was just cancelled in order not to block program execuiton (use with precaution!)
	 */
	bool SetNonBlocking();

	/*
	 *	bool CSocket::b_WouldBlock() const
	 *		- returns true in case EWOULDBLOCK (or WSAEWOULDBLOCK under win) was set, otherwise false
	 *		- this flag means that recently failed socket operation didn't fail because of real error
	 *		  but because performing it would mean blocking program execution (non-blocking sockets)
	 */
	bool b_WouldBlock() const;
};

/*
 *	class CStreamSocket
 *		- class for simple stream (TCP/IP) socket operation
 */
class CStreamSocket : public CSocket {
public:
	/*
	 *	CStreamSocket::CStreamSocket()
	 *		- default constructor, creates socket (call b_State() to determin wheter successfuly)
	 */
	CStreamSocket();

	/*
	 *	bool CStreamSocket::Create()
	 *		- creates socket
	 *		- returns true in case socket is created in time of execution
	 *		  leaving this function, otherwise false
	 */
	bool Create();

	/*
	 *	bool CStreamSocket::Connect(const TAddress &r_t_address)
	 *		- attempts to connect to host given by r_t_address
	 *		- returns true in case connection was established, otherwise false
	 *		- always returns false in case no socket was created
	 */
	bool Connect(const TAddress &r_t_address);

	/*
	 *	bool CStreamSocket::Listen(const TAddress &r_t_address, int n_max_connections)
	 *		- binds socket to address r_t_address, begin listening for n_max_connections
	 *		- returns true in case socket is listening, otherwise false
	 *		- always returns false in case no socket was created
	 */
	bool Listen(const TAddress &r_t_address, int n_max_connections);

	/*
	 *	CStreamSocket *CStreamSocket::p_Accept(TAddress &r_t_address) const
	 *		- accepts incoming connections; r_t_address is filled with network address of socket
	 *		  which is returned
	 *		- returns a new socket, connected to the other side or 0 on failure
	 *		- note it propably will return 0 in case Listen() was not successfuly called prior calling this function
	 *		- always returns 0 in case no socket was created
	 */
	CStreamSocket *p_Accept(TAddress &r_t_address) const;

	/*
	 *	int CStreamSocket::n_Read(void *p_buffer, int n_buffer_size) const
	 *		- receives data from socket to buffer
	 *		- maximum amount of data read is given by n_buffer_size but it can also read less data
	 *		- returns number of bytes written into the buffer on success or -1 on failure
	 *		- always returns -1 in case no socket was created
	 *		- note in case there are no data for the socket, the function is by default waiting
	 *		  until some arrives (can be overriden by calling SetNonBlocking())
	 */
	int n_Read(void *p_buffer, int n_buffer_size) const;

	/*
	 *	int CStreamSocket::n_Write(const void *p_buffer, int n_buffer_size) const
	 *		- transmits n_buffer_size bytes from buffer to socket
	 *		- returns number of bytes transmitted into socket on success or -1 on failure
	 *		- always returns -1 in case no socket was created
	 *		- note in case the socket buffer is full or there is another reason why the
	 *		  operation, canot be performed immediately the function is by default waiting
	 *		  until it can proceed (can be overriden by calling SetNonBlocking())
	 */
	int n_Write(const void *p_buffer, int n_buffer_size) const;
};

/*
 *	class CDatagramSocket
 *		- class for simple UDP socket operation
 */
class CDatagramSocket : public CSocket {
public:
	/*
	 *	CDatagramSocket::CDatagramSocket()
	 *		- default constructor, creates socket (call b_State() to determin wheter successfuly)
	 */
	CDatagramSocket();

	/*
	 *	bool CDatagramSocket::Create()
	 *		- creates socket
	 *		- returns true in case socket is created in time of execution
	 *		  leaving this function, otherwise false
	 */
	bool Create();

	/*
	 *	bool CDatagramSocket::Bind(const TAddress &r_t_address)
	 *		- binds the socket to address r_t_address
	 *		- returns true on succes or false on failure
	 *		- always returns false in case no socket was created
	 */
	bool Bind(const TAddress &r_t_address);

	/*
	 *	int CDatagramSocket::n_SendTo(const TAddress &r_t_address, const void *p_buffer, int n_buffer_size) const
	 *		- sends n_buffer_size bytes to  socket from buffer
	 *		- returns number of bytes written into socket on success or -1 on failure
	 *		- always returns -1 in case no socket was created
	 *		- note in case the socket buffer is full or there is another reason why the
	 *		  operation, canot be performed immediately the function is by default waiting
	 *		  until it can proceed (can be overriden by calling SetNonBlocking())
	 */
	int n_SendTo(const TAddress &r_t_address, const void *p_buffer, int n_buffer_size) const;

	/*
	 *	int CDatagramSocket::n_RecvFrom(const TAddress &r_t_address, void *p_buffer, int n_buffer_size) const
	 *		- receives n_buffer_size bytes from socket to buffer; r_t_address is set to address which sent the data
	 *		- returns number of bytes written into socket on success or -1 on failure
	 *		- always returns -1 in case no socket was created
	 *		- note in case the socket buffer is full or there is another reason why the
	 *		  operation, canot be performed immediately the function is by default waiting
	 *		  until it can proceed (can be overriden by calling SetNonBlocking())
	 */
	int n_RecvFrom(TAddress &r_t_address, void *p_buffer, int n_buffer_size) const;
};

#endif //__SOCKET_INCLUDED
